from abc import abstractmethod
from typing import Callable, Coroutine, Type
from arkitekt.messages.base import MessageModel
from herre.config.base import BaseConfig
from herre.auth import get_current_herre
from enum import Enum
import asyncio

class TransportConfig(BaseConfig):
    _group = "arkitekt.postman"
    pass

class Transport(object):


    def __init__(self, broadcast = None) -> None:
        self._broadcast = broadcast

        assert self._broadcast is not None, "Please provide a broadcaster that receives messages"

        self.herre = get_current_herre()
        self.loop = get_current_herre().loop


    @abstractmethod
    def connect(self):
        raise NotImplementedError()


    @abstractmethod
    def disconnect(self):
        raise NotImplementedError()

    async def broadcast(self, message: MessageModel):
        return await self._broadcast(message)


    @abstractmethod
    async def forward(self, message: MessageModel):
        raise NotImplementedError()






from ....messages.exception import ExceptionDataModel, ExceptionMessage
from pydantic.main import BaseModel
from ....messages.types import  PROVIDE, PROVIDE_CRITICAL, PROVIDE_DONE, RESERVE_CRITICAL
from ....messages.base import MessageDataModel, MessageMetaExtensionsModel, MessageMetaModel, MessageModel
from typing import List, Optional


class MetaExtensionsModel(MessageMetaExtensionsModel):
    # Set by postman consumer
    progress: Optional[str]
    callback: Optional[str]

class MetaModel(MessageMetaModel):
    type: str = RESERVE_CRITICAL
    extensions: Optional[MetaExtensionsModel]

class ReserveCriticalMessage(ExceptionMessage):
    data: ExceptionDataModel
    meta: MetaModel


from pydantic.main import BaseModel
from ....messages.types import  PROVIDE, UNPROVIDE
from ....messages.base import MessageDataModel, MessageMetaExtensionsModel, MessageMetaModel, MessageModel
from typing import List, Optional

class ProvideParams(BaseModel):
    providers: Optional[List[str]]


class ProvideMetaExtensionsModel(MessageMetaExtensionsModel):
    # Set by postman consumer
    progress: Optional[str]
    callback: Optional[str]

class ProvideMetaModel(MessageMetaModel):
    type: str = UNPROVIDE
    extensions: Optional[ProvideMetaExtensionsModel]

class DataModel(MessageDataModel):
    provision: str


class UnprovideMessage(MessageModel):
    data: DataModel
    meta: ProvideMetaModel


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['arkitekt',
 'arkitekt.actors',
 'arkitekt.agents',
 'arkitekt.contracts',
 'arkitekt.graphql',
 'arkitekt.legacy',
 'arkitekt.messages',
 'arkitekt.messages.app',
 'arkitekt.messages.postman',
 'arkitekt.messages.postman.assign',
 'arkitekt.messages.postman.provide',
 'arkitekt.messages.postman.reserve',
 'arkitekt.messages.postman.unassign',
 'arkitekt.messages.postman.unprovide',
 'arkitekt.messages.postman.unreserve',
 'arkitekt.monitor',
 'arkitekt.packers',
 'arkitekt.packers.models',
 'arkitekt.packers.transpilers',
 'arkitekt.policies',
 'arkitekt.schema',
 'arkitekt.transport',
 'arkitekt.ui',
 'arkitekt.ui.qtwidgets']

package_data = \
{'': ['*']}

install_requires = \
['docstring-parser>=0.10,<0.11',
 'herre>=0.1.19,<0.2.0',
 'inflection>=0.5.1,<0.6.0',
 'janus>=0.6.1,<0.7.0',
 'websockets>=10.0,<11.0']

setup_kwargs = {
    'name': 'arkitekt',
    'version': '0.1.23',
    'description': 'rpc and node backbone',
    'long_description': None,
    'author': 'jhnnsrs',
    'author_email': 'jhnnsrs@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
